<?php

namespace EventPortal\Http\Controllers\Admin\Event;

use EventPortal\Domain\Event\Actions\DeleteEvent;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Form\Facades\Form;
use EventPortal\Domain\Table\Facades\Table;
use EventPortal\Domain\Util\Breadcrumb;
use EventPortal\Http\Controllers\Controller;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Inertia\Inertia;
use Inertia\Response;

class EventController extends Controller
{
    public function __construct()
    {
        $this->middleware('check-permission:event.show')->only('show');
        $this->middleware('check-permission:event.store')->only(['create', 'store']);
        $this->middleware('check-permission:event.update')->only(['edit', 'update']);
        $this->middleware('check-permission:event.destroy')->only('delete');
    }

    public function index(): Response
    {
        $table = Table::make('event.index')
            ->query(Event::query()->isNotArchived())
            ->request(request())
            ->get();

        return Inertia::render('Event/Index', [
            'data' => $table,
        ]);
    }

    public function create(): Response
    {
        $form = Form::make('event.create')->model(new Event)->render();

        $breadcrumbs = Breadcrumb::make()
            ->add('Events', route('admin.event.index'))
            ->add('Create an event')
            ->get();

        return Inertia::render('Event/Create', [
            'form' => $form,
            'breadcrumbs' => $breadcrumbs,
        ]);
    }

    public function store(Request $request): RedirectResponse
    {
        Form::make('event.create')
            ->model(new Event)
            ->validate($request->all())
            ->save($request->all());

        return redirect()->route('admin.event.index');
    }

    public function edit(Event $event): Response
    {
        $form = Form::make('event.edit')->model($event)->render();

        $breadcrumbs = Breadcrumb::make()
            ->add('Events', route('admin.event.index'))
            ->add($event->name, route('admin.event.show', $event->id))
            ->add('Edit event')
            ->get();

        return Inertia::render('Event/Edit', [
            'form' => $form,
            'breadcrumbs' => $breadcrumbs,
            'eventName' => $event->name,
        ]);
    }

    public function update(Request $request, Event $event): RedirectResponse
    {
        Form::make('event.edit')
            ->model($event)
            ->validate($request->all())
            ->save($request->all());

        return redirect()->route('admin.event.index');
    }

    public function show(Event $event): Response
    {
        $form = Form::make('event.show')->model($event)->render();

        $breadcrumbs = Breadcrumb::make()
            ->add('Events', route('admin.event.index'))
            ->add($event->name)
            ->get();

        return Inertia::render('Event/Show', [
            'data' => $form,
            'breadcrumbs' => $breadcrumbs,
            'eventName' => $event->name,
        ]);
    }

    public function delete(Event $event, DeleteEvent $deleteEvent): RedirectResponse
    {
        $deleteEvent->execute($event);

        return redirect()->route('admin.event.index');
    }
}
